@@ -22,9 +22,10 @@ where the concrete classes and logic are for those purposes.
 
 import base64
 import io
+import json
 import logging
 import re
-from typing import List, Dict
+from typing import Dict, List
 import urllib.parse
 import zipfile
 
@@ -90,7 +91,8 @@ class IKS(BaseClient):
     def __init__(self, *args, **kwargs):
         super(IKS, self).__init__(*args, **kwargs)
 
-        # IKS likes to throw back random 503s at times, but retrying generally works fine
+        # IKS likes to throw back random 503s at times,
+        # but retrying generally works fine
         # requests default is Retry(0, read=False); see requests/adapters.py
         retry_conf = Retry(
             total=5, read=False, backoff_factor=1, status_forcelist=[502, 503]
@@ -242,13 +244,13 @@ class IKS(BaseClient):
         # this is a pretty hacky way of getting one, but we shouldn't even
         # need it either so... idk. /shrug
 
-        # do a get token before, to make sure we get a valid refresh token.
-        _token = self.session.auth._token_manager.get_token()
+        # do a get token before, to make sure we get a valid refresh token
+        _ = self.session.auth._token_manager.get_token()
         refresh_token = self.session.auth._token_manager._token_info.get(
             "refresh_token"
         )
 
-        # pure yaml output was added sometime after our original code to deal with the zipfile,
+        # yaml output was added after our original code to deal with the zipfile
         # leaving both in because maybe its useful for someone to use the zip path still
         params = {}
         if output_format == "yaml":
@@ -299,7 +301,9 @@ class ResourceController(BaseClient):
     """
 
     names = ["rc"]
-    KEYPROTECT_PLAN_ID = "eedd3585-90c6-4c8f-be3d-062069e99fc3"  # keyprotect tiered-pricing ID
+    KEYPROTECT_PLAN_ID = (
+        "eedd3585-90c6-4c8f-be3d-062069e99fc3"  # keyprotect tiered-pricing ID
+    )
 
     def __init__(self, *args, **kwargs):
         super(ResourceController, self).__init__(*args, **kwargs)
@@ -343,7 +347,9 @@ class ResourceController(BaseClient):
 
         # apparently it doesn't complain if we drop query params,
         # didn't want to have to look up the account ID anyway, so +2
-        resp = self.session.get("{0}/v2/resource_groups".format(netloc),)
+        resp = self.session.get(
+            "{0}/v2/resource_groups".format(netloc),
+        )
 
         if resp.status_code != 200:
             raise Exception(
@@ -396,9 +402,12 @@ class ResourceController(BaseClient):
 
     def _create_instance_v1(self, name, region, resource_group_id, resource_plan_id):
 
-        # seems like the target_crn is the region selector, and its just the price plan ID with the region stuck at the end
-        target_crn = "crn:v1:bluemix:public:globalcatalog::::deployment:{0}%3A{1}".format(
-            resource_plan_id, region
+        # seems like the target_crn is the region selector,
+        # and its just the price plan ID with the region stuck at the end
+        target_crn = (
+            "crn:v1:bluemix:public:globalcatalog::::deployment:{0}%3A{1}".format(
+                resource_plan_id, region
+            )
         )
 
         body = {
@@ -436,7 +445,7 @@ class ResourceController(BaseClient):
 
     def list_instances(self):
         """
-        Retrieve a list of all the service and resource instances in the current account.
+        Retrieve a list of all service and resource instances in the current account.
 
         Note this will return an iterator that will handle the underlying pagination of
         large sets of instances returned.
@@ -474,12 +483,13 @@ class KeyProtect(BaseClient):
     """
     API Docs: https://cloud.ibm.com/apidocs/key-protect
     """
+
     class KeyProtectError(Exception):
         @staticmethod
         def wrap(http_error):
             try:
                 message = http_error.response.json()["resources"][0]["errorMsg"]
-            except KeyError:
+            except (KeyError, json.decoder.JSONDecodeError, ValueError):
                 message = http_error.response.text
             err = KeyProtect.KeyProtectError(message)
             err.http_error = http_error
@@ -521,21 +531,21 @@ class KeyProtect(BaseClient):
             http_err.raw_response = log_resp(resp)
             raise KeyProtect.KeyProtectError.wrap(http_err)
 
-    def keys(self):
+    def list_keys(self):
         resp = self.session.get("%s/api/v2/keys" % self.endpoint_url)
 
         self._validate_resp(resp)
 
         return resp.json().get("resources", [])
 
-    def get(self, key_id):
+    def get_key(self, key_id):
         resp = self.session.get("%s/api/v2/keys/%s" % (self.endpoint_url, key_id))
 
         self._validate_resp(resp)
 
         return resp.json().get("resources")[0]
 
-    def create(self, name, payload=None, raw_payload=None, root=False):
+    def create_key(self, name, payload=None, raw_payload=None, root=False):
 
         data = {
             "metadata": {
@@ -561,7 +571,7 @@ class KeyProtect(BaseClient):
         self._validate_resp(resp)
         return resp.json().get("resources")[0]
 
-    def delete(self, key_id):
+    def delete_key(self, key_id):
         resp = self.session.delete("%s/api/v2/keys/%s" % (self.endpoint_url, key_id))
         self._validate_resp(resp)
 
@@ -598,13 +608,311 @@ class KeyProtect(BaseClient):
         resp = self._action(key_id, "unwrap", data)
         return base64.b64decode(resp["plaintext"].encode("utf-8"))
 
-    def rotate(self, key_id, payload=None):
+    def rotate_key(self, key_id, payload=None):
         data = None
         if payload:
             data = {"payload": base64.b64encode(payload).decode("utf-8")}
 
         return self._action(key_id, "rotate", data)
 
+    def disable_key(self, key_id: str):
+        """
+        Disable a key.
+
+        The key will not be deleted, but it will not be active and
+        key operations cannot be performed on a disabled key.
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#disablekey
+        """
+        resp = self.session.post(
+            "%s/api/v2/keys/%s/actions/disable" % (self.endpoint_url, key_id)
+        )
+        self._validate_resp(resp)
+
+    def enable_key(self, key_id: str):
+        """
+        Enable a key.
+
+        Only disabled keys can be enabled. After calling this action, the
+        key becomes active and key operations can be performed on it.
+
+        Note: This does not recover Deleted keys.
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#enablekey
+        """
+        resp = self.session.post(
+            "%s/api/v2/keys/%s/actions/enable" % (self.endpoint_url, key_id)
+        )
+        self._validate_resp(resp)
+
+    def create_key_ring(self, key_ring_id: str):
+        """
+        Create a key ring in the instance with the specified name.
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#createkeyring
+        """
+
+        resp = self.session.post(
+            "%s/api/v2/key_rings/%s" % (self.endpoint_url, key_ring_id),
+        )
+        self._validate_resp(resp)
+
+    def get_key_rings(self):
+        """
+        Get all key rings associated with specified instance
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#listkeyrings
+        """
+        resp = self.session.get(
+            "%s/api/v2/key_rings" % self.endpoint_url,
+        )
+        self._validate_resp(resp)
+        return resp.json()
+
+    def delete_key_ring(self, key_ring_id: str):
+        """
+        Deletes a key ring from the associated instance
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#deletekeyring
+        """
+        resp = self.session.delete(
+            "%s/api/v2/key_rings/%s" % (self.endpoint_url, key_ring_id),
+        )
+        self._validate_resp(resp)
+
+    def create_import_token(
+        self, expiration: int = None, max_allowed_retrievals: int = None
+    ):
+        """
+        Create an import token that can be used to import encrypted material as root keys.
+
+        expiration: The time in seconds from the creation of a import token that determines how long it remains valid.
+            The minimum value is 300 seconds (5 minutes), and the maximum value is 86400 (24 hours). The default value
+            is 600 (10 minutes).
+        max_allowed_retrievals: The number of times that an import token can be retrieved within its expiration time
+            before it is no longer accessible. The default value is 1. The maximum value is 500.
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#postimporttoken
+        """
+
+        data = {}
+        if expiration:
+            data["expiration"] = expiration
+        if max_allowed_retrievals:
+            data["maxAllowedRetrievals"] = max_allowed_retrievals
+        resp = self.session.post(
+            "%s/api/v2/import_token" % self.endpoint_url, json=data
+        )
+
+        self._validate_resp(resp)
+        return resp.json()
+
+    def get_registrations(self, key_id: str = None, crn: str = None):
+        """
+        Retrieve a list of registrations
+
+        If `key_id` is None (the default) all registrations for the instance are
+        returned, otherwise only the registrations associated with a specified root
+        key are returned.
+
+        `crn` should be a str type that will be passed as the `urlEncodedResourceCRNQuery` parameter to the HTTP API.
+        It is used to filter registration on a specific cloud resource. More information can be found in the API docs below.
+
+        API Docs:
+          - https://cloud.ibm.com/apidocs/key-protect#getregistrations
+          - https://cloud.ibm.com/apidocs/key-protect#getregistrationsallkeys
+        """
+        params = {}
+        if crn is not None:
+            params["urlEncodedResourceCRNQuery"] = crn
+
+        if key_id is not None:
+            url = "%s/api/v2/keys/%s/registrations" % (self.endpoint_url, key_id)
+        else:
+            url = "%s/api/v2/keys/registrations" % self.endpoint_url
+
+        resp = self.session.get(url, params=params)
+
+        self._validate_resp(resp)
+        return resp.json()
+
+    def get_import_token(self):
+        """
+        Retrieves an import token associated with the current service instance. Token must be previously created by
+        a create import token call.
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#getimporttoken
+        """
+
+        resp = self.session.get("%s/api/v2/import_token" % self.endpoint_url)
+        self._validate_resp(resp)
+        return resp.json()
+
+    def create_key_alias(self, key_id: str, alias: str):
+        """
+        Creates an alias name for a key.
+        An alias is a user defined string that can be used in place of a normal UUID Key ID
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#createkeyalias
+        """
+
+        resp = self.session.post(
+            "%s/api/v2/keys/%s/aliases/%s" % (self.endpoint_url, key_id, alias)
+        )
+        self._validate_resp(resp)
+        return resp.json()
+
+    def delete_key_alias(self, key_id: str, alias: str):
+        """
+        Deletes an alias name associated with a key
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#deletekeyalias
+        """
+
+        resp = self.session.delete(
+            "%s/api/v2/keys/%s/aliases/%s" % (self.endpoint_url, key_id, alias)
+        )
+        self._validate_resp(resp)
+
+    def _set_policy(self, resources_list, key_id=None):
+
+        collection_total = len(resources_list)
+        data = {
+            "metadata": {
+                "collectionType": "application/vnd.ibm.kms.policy+json",
+                "collectionTotal": collection_total,
+            },
+            "resources": resources_list,
+        }
+
+        if key_id:
+            resp = self.session.put(
+                "%s/api/v2/keys/%s/policies" % (self.endpoint_url, key_id), json=data
+            )
+        else:
+            resp = self.session.put(
+                "%s/api/v2/instance/policies" % self.endpoint_url, json=data
+            )
+
+        self._validate_resp(resp)
+        if resp.status_code == 204:
+            return
+        else:
+            return resp.json()
+
+    def set_key_rotation_policy(self, key_id: str, rotation_interval: int):
+        """
+        Updates the rotation policy associated with a key by specifying key ID and rotation interval
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#putpolicy
+        """
+        resources_list = [
+            {
+                "type": "application/vnd.ibm.kms.policy+json",
+                "rotation": {
+                    "interval_month": int(rotation_interval),
+                },
+            }
+        ]
+        return self._set_policy(resources_list, key_id)
+
+    def set_key_dual_auth_policy(self, key_id: str, dual_auth_enable: bool):
+        """
+        Updates the dual auth delete policy by passing the key ID and enable detail
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#putpolicy
+        """
+        resources_list = [
+            {
+                "type": "application/vnd.ibm.kms.policy+json",
+                "dualAuthDelete": {"enabled": dual_auth_enable},
+            }
+        ]
+        return self._set_policy(resources_list, key_id)
+
+    def get_key_policies(self, key_id: str):
+        """
+        Retrieves a list of policies that are associated with a specified key
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#getpolicy
+        """
+        resp = self.session.get(
+            "%s/api/v2/keys/%s/policies" % (self.endpoint_url, key_id)
+        )
+        self._validate_resp(resp)
+        return resp.json()
+
+    def set_instance_dual_auth_policy(self, dual_auth_enable: bool):
+        """
+        Updates the dual auth delete policy for the instance by passing the enable detail
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#putinstancepolicy
+        """
+        resources_list = [
+            {
+                "policy_type": "dualAuthDelete",
+                "policy_data": {"enabled": dual_auth_enable},
+            }
+        ]
+        return self._set_policy(resources_list)
+
+    def set_instance_allowed_network_policy(
+        self, allowed_network_enable: bool, network_type: str
+    ):
+        """
+        Updates the allowed network policy for the instance
+
+        `network_type` is a str type, and must be one of "public-and-private" or "private-only".
+        The default is "public-and-private", but can be set to "private-only" to disable access to the instance from Internet client addresses.
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#putinstancepolicy
+        """
+        resources_list = [
+            {
+                "policy_type": "allowedNetwork",
+                "policy_data": {
+                    "enabled": allowed_network_enable,
+                    "attributes": {"allowed_network": network_type},
+                },
+            }
+        ]
+        return self._set_policy(resources_list)
+
+    def set_instance_allowed_ip_policy(
+        self, allowed_ip_enable: bool, allowed_ips: List[str]
+    ):
+        """
+        Updates the allowed IP policy for the instance
+
+        API Docs: https://cloud.ibm.com/apidocs/key-protect#putinstancepolicy
+        """
+        resources_list = [
+            {
+                "policy_type": "allowedIP",
+                "policy_data": {
+                    "enabled": allowed_ip_enable,
+                    "attributes": {"allowed_ip": allowed_ips},
+                },
+            }
+        ]
+        return self._set_policy(resources_list)
+
+    def get_instance_policies(self):
+        """
+        Retrieves a list of policies that are associated with a specified service instance
+
+        https://cloud.ibm.com/apidocs/key-protect#getinstancepolicy
+        """
+        resp = self.session.get("%s/api/v2/instance/policies" % (self.endpoint_url))
+        self._validate_resp(resp)
+        return resp.json()
+
+    # deprecated methods
+    keys = list_keys
+    get = get_key
+    create = create_key
+    delete = delete_key
+    rotate = rotate_key
+
 
 class CISAuth(requests.auth.AuthBase):
     def __init__(self, credentials):
